@@ -12,8 +12,8 @@ try:
 except ImportError:
     import queue as Queue
 
-__version__ = "0.5.0"
-__version_info__ = ( 0, 5, 0)
+__version__ = "0.5.2"
+__version_info__ = ( 0, 5, 2)
 
 class _LoggerObject(object):
     def __init__(self,logname='cmdpack'):
@@ -400,7 +400,12 @@ class _CmdRunObject(_LoggerObject):
 
     def __wait_recvq(self):
         if self.recvq is not None:
-            assert(self.recvq.empty())
+            while not self.recvq.empty():
+                try:
+                    rl = self.recvq.get_nowait()
+                    self.info('get [%s]'%(rl))
+                except Queue.Empty:
+                    pass
             # nothing to be done
             self.recvq = None
         return
@@ -418,6 +423,25 @@ class _CmdRunObject(_LoggerObject):
                 # wait for a time
                 self.info('will wait')
                 time.sleep(0.1)
+            while (not self.errended) or (not self.outended):
+                if not self.errended:
+                    try:
+                        rl = self.enderr.get_nowait()
+                        if rl == 'done':
+                            self.errended = True
+                            self.enderr.join()
+                            self.enderr = None
+                    except Queue.Empty:
+                        pass
+                if not self.outended:
+                    try:
+                        rl = self.endout.get_nowait()
+                        if rl == 'done':
+                            self.outended = True
+                            self.endout.join()
+                            self.endout = None
+                    except Queue.Empty:
+                        pass
             if self.__p.stdout is not None:
                 self.__p.stdout.close()
                 self.__p.stdout = None
@@ -538,11 +562,44 @@ class _CmdRunObject(_LoggerObject):
                     if not self.errended or not self.outended:
                         # sleep for a while to get 
                         time.sleep(0.1)
+        else:
+            notempty = True
+            while len(retlines) < minlines and notempty:
+                try:
+                    if self.recvq is not None:
+                        rl = self.recvq.get_nowait()
+                        retlines.append(rl)
+                    else:
+                        notempty = False
+                except Queue.Empty:
+                    notempty = False
+                    if not self.outended:
+                        try:
+                            rl = self.endout.get_nowait()
+                            if rl == 'done':
+                                self.outended = True
+                                self.endout.join()
+                                self.endout = None
+                        except Queue.Empty:
+                            pass
+                    if not self.errended:
+                        try:
+                            rl = self.enderr.get_nowait()
+                            if rl == 'done':
+                                self.errended = True
+                                self.enderr.join()
+                                self.enderr = None
+                        except Queue.Empty:
+                            pass
         return retlines
 
 
 
     def __clean_resource(self):
+        while True:
+            rlines = self.get_lines(0.1,100)
+            if len(rlines) == 0:
+                break
         self.__wait_out()
         self.__wait_err()
         self.__wait_recvq()
@@ -585,47 +642,56 @@ class _CmdRunObject(_LoggerObject):
             while True:
                 if self.errended and self.outended:
                     break
-                try:
-                    rl = self.recvq.get_nowait()
-                    self.info('rl (%s)'%(rl.rstrip('\r\n')))
-                except Queue.Empty:
-                    if not self.errended:
-                        try:
-                            rl = self.enderr.get_nowait()
-                            if rl == 'done':
-                                self.errended = True
-                                self.enderr.join()
-                                self.enderr = None
-                        except Queue.Empty:
-                            pass
-                    if not self.outended :
-                        try:
-                            rl = self.endout.get_nowait()
-                            if rl == 'done':
-                                self.outended = True
-                                self.endout.join()
-                                self.endout = None
-                        except Queue.Empty:
-                            pass
-                    if not self.errended or not self.outended:
-                        # sleep for a while to get 
-                        if maxwtime is not None:
-                            ctime = time.time()
-                            if (ctime - stime) > maxwtime:
-                                self.info('[%s] kill[%s]'%(ctime,self.__p.pid))
-                                self.__kill_proc_childs(self.__p.pid)
-                        time.sleep(0.1)
+                if not self.errended:
+                    try:
+                        rl = self.enderr.get_nowait()
+                        if rl == 'done':
+                            self.errended = True
+                            self.enderr.join()
+                            self.enderr = None
+                    except Queue.Empty:
+                        pass
+                if not self.outended:
+                    try:
+                        rl = self.endout.get_nowait()
+                        if rl == 'done':
+                            self.outended = True
+                            self.endout.join()
+                            self.endout = None
+                    except Queue.Empty:
+                        pass
+                if not self.errended or not self.outended:
+                    # sleep for a while to get 
+                    if maxwtime is not None:
+                        ctime = time.time()
+                        if (ctime - stime) > maxwtime:
+                            self.info('[%s] kill[%s]'%(ctime,self.__p.pid))
+                            self.__kill_proc_childs(self.__p.pid)
+                    time.sleep(0.1)
         self.__retcode = exitcode
         return exitcode
 
     def __del__(self):
         # we do not clean_resource because on here we do not any more
+        attr = CmdObjectAttr()
+        attr.maxwtime = 0.01
+        self.__kill_proc(attr)
+        self.__clean_resource()
         return
 
     def get_exitcode(self,attr=None):
         self.__kill_proc(attr)
         return self.__clean_resource()
 
+    def is_running(self):
+        if self.__p is not None:
+            pret = self.__p.poll()
+            if pret is not None:
+                self.__get_exitcode()
+                return False
+            return True
+        return False
+
 
 def run_command_callback(cmd,callback,ctx,stdoutfile=subprocess.PIPE,stderrfile=None,shellmode=True,copyenv=None,linebuf=True):
     cmdobj = _CmdRunObject(cmd,stdoutfile,stderrfile,shellmode,copyenv,[],linebuf)
